%% Copyright (C) 2021 Tony Richardson
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <https://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {} {@var{[B, A]} =} lp2bs (@var{Bp}, @var{Ap}, @var{w0}, @var{W))
%% Transform a lowpass s-plane filter into an s-plane bandstop filter.
%% Bp and Ap are numerator and denominator coefficients of the lowpass
%% prototype filter.  w0 and W are the center frequency and bandwidth
%% (rad/s) of the desired bandstop filter.  The numerator and denominator
%% coefficients of the desired bandstop filter are returned.
%%
%% @seealso{lp2lp, lp2hp, lp2bp}
%% @end deftypefn

%% Author: Tony Richardson <richardson.tony@gmail.com>
%% Created: 2021-04-09

function [B, A] = lp2bs(Bp, Ap, w0, W)
  [z p k] = tf2zp(Bp, Ap);
  [z p k] = sftrans(z, p, k, w0 + [-W/2 W/2], true);
  [B A] = zp2tf(z, p, k);
end

%!demo
%! n = 10;
%! [z p k] = buttap(n);
%! [b a] = zp2tf(z, p, real(k));
%!
%! w = logspace(-1, 1, 501);
%! H = freqs(b, a, w);
%! figure(1)
%! clf
%! subplot(2, 1, 1)
%! loglog(w, abs(H))
%! ylim([1e-15 1]);
%! yticks(logspace(-15, 0, 4))
%! grid on
%! subplot(2, 1, 2)
%! semilogx(w, rad2deg(angle(H)))
%! grid on
%!
%! fl = 20; fh = 60;
%! Wo = 2*pi*sqrt(fl*fh);
%! Bw = 2*pi*(fh - fl);
%! [bt, at] = lp2bs(b, a, Wo, Bw);
%!
%! w = logspace(1, 3, 501);
%! H = freqs(bt, at, w);
%!
%! figure(2)
%! clf
%! subplot(2, 1, 1)
%! loglog(w, abs(H))
%! ylim([1e-20 1]);
%! yticks(logspace(-20, 0, 3))
%! grid on
%! subplot(2, 1, 2)
%! semilogx(w, rad2deg(angle(H)))
%! grid on
